import datetime
import json
import logging
import sys
from functools import wraps
from threading import Thread

import six
from dateutil.parser import parse as parse_date


def attrs_to_dir(mod):
    data = {}
    for attr in dir(mod):
        if attr == attr.upper():
            data[attr] = getattr(mod, attr)
    return data


class Task(Thread):
    """
    The Threaded object returned by the @threaded decorator below
    """

    def __init__(self, method, *args, **kwargs):
        super(Task, self).__init__()
        self.method = method
        self.name = 'ETCDConfigThread'
        self.args = args
        self.kwargs = kwargs
        self._result = None
        self.__exc_info = None

    def run(self):
        try:
            self._result = self.method(*self.args, **self.kwargs)
        except:
            self.__exc_info = sys.exc_info()

    @property
    def result(self):
        self.join()
        if self.__exc_info is not None:
            six.reraise(*self.__exc_info)
        return self._result


def threaded(function=None, daemon=False):

    def wrapper_factory(func):

        @wraps(func)
        def get_thread(*args, **kwargs):
            t = Task(func, *args, **kwargs)
            if daemon:
                t.daemon = True
            t.start()
            return t

        return get_thread

    if function:
        return wrapper_factory(function)
    else:
        return wrapper_factory


class CustomJSONEncoder(json.JSONEncoder):

    custom_type_key = '_custom_type'
    custom_type_value_key = 'value'
    DECODERS = {'datetime': parse_date}

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return {self.custom_type_key: 'datetime',
                    self.custom_type_value_key: obj.isoformat()}
        else:
            return super(CustomJSONEncoder, self).default(obj)


def custom_json_decoder_hook(obj):

    ct = obj.get(CustomJSONEncoder.custom_type_key, None)
    if ct is not None:
        value = obj.get(CustomJSONEncoder.custom_type_value_key)
        return CustomJSONEncoder.DECODERS[ct](value)
    else:
        return obj


def byteify(input):
    if isinstance(input, dict):
        return {byteify(key): byteify(value) for key, value in input.items()}
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif (sys.version_info.major == 2) and (isinstance(input, unicode)):
        return input.encode('utf-8')
    else:
        return input


class IgnoreMaxEtcdRetries(logging.Filter):
    """
    Skip etcd.client MaxRetryError on timeout
    """

    def __init__(self, name='etcd.client'):
        super(IgnoreMaxEtcdRetries, self).__init__(name)

    def filter(self, record):
        msg = '{}'.format(record.args)
        return not (
            'MaxRetryError' in msg and
            'Read timed out' in msg
        )




from setuptools import setup
setup(**{'author': 'Andrey Makhnach',
 'author_email': 'andrey.makhnach@kpn.com',
 'classifiers': ['Development Status :: 5 - Production/Stable',
                 'Environment :: Web Environment',
                 'Intended Audience :: Developers',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Topic :: Internet :: WWW/HTTP'],
 'description': 'A dynamic settings management solution using ETCD',
 'include_package_data': True,
 'install_requires': ['python-etcd>=0.4.1',
                      'python-dateutil>=2.2',
                      'six<2.0.0,>=1.10.0'],
 'long_description': "etcd_config\n===========\n\n.. image:: https://secure.travis-ci.org/kpn-digital/etcd_config.svg?branch=master\n    :target:  http://travis-ci.org/kpn-digital/etcd_config?branch=master\n\n.. image:: https://img.shields.io/codecov/c/github/kpn-digital/etcd_config/master.svg\n    :target: http://codecov.io/github/kpn-digital/etcd_config?branch=master\n\n.. image:: https://img.shields.io/pypi/v/etcd_config.svg\n    :target: https://pypi.python.org/pypi/etcd_config\n\n.. image:: https://readthedocs.org/projects/etcd_config/badge/?version=latest\n    :target: http://etcd_config.readthedocs.org/en/latest/?badge=latest\n\n\nFeatures\n--------\n\nThis library allows Python applications load configuration from ETCD:\n\n* Environment dependent values\n* Values in different config sets, identified by name\n\n\nBackends\n--------\n\n- ETCD 2.2.1\n\n\nInstallation\n------------\n\n.. code-block:: bash\n\n    $ pip install etcd-config\n\n\nUsage\n-----\n\n.. code-block:: python\n\n    import etcd_config.loader\n    config = etcd_config.loader.get_overwrites(\n        env='test',\n        dev_params='main.params',\n        etcd_details=dict(\n            protocol=getattr(params, 'ETCD_PROTOCOL', 'http'),\n            host=getattr(params, 'ETCD_HOST', 'localhost'),\n            port=getattr(params, 'ETCD_PORT', 2379),\n            username=getattr(params, 'ETCD_USERNAME', None),\n            password=getattr(params, 'ETCD_PASSWORD', None),\n            prefix='/config/your_project'\n        )\n    )\n",
 'name': 'etcd-config',
 'packages': ['etcd_config', 'tests'],
 'tests_require': ['tox'],
 'url': 'https://github.com/kpn-digital/py-etcd-config',
 'version': u'1.0.3',
 'zip_safe': False})
